@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 1999,2000	Jens Axboe <axboe@suse.de>
  * Copyright (c) 2004		Peter Osterlund <petero2@telia.com>
+ * Copyright (c) 2014-2017	Pali Rohár <pali.rohar@gmail.com>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -71,12 +72,12 @@ struct pkt_ctrl_command {
 static int init_cdrom(int fd)
 {
 	if (ioctl(fd, CDROM_DRIVE_STATUS, CDSL_CURRENT) < 0) {
-		perror("drive not ready\n");
+		fprintf(stderr, "pktsetup: Error: Drive is not ready\n");
 		return 1;
 	}
 
 	if (ioctl(fd, CDROM_DISC_STATUS, CDSL_CURRENT) < 0) {
-		perror("no disc inserted?\n");
+		fprintf(stderr, "pktsetup: Error: No disc inserted\n");
 		return 1;
 	}
 
@@ -93,14 +94,14 @@ static int setup_dev(char *pkt_device, char *device, int rem)
 	int pkt_fd, dev_fd, cmd, arg, ret;
 
 	if ((pkt_fd = open(pkt_device, O_RDONLY)) == -1) {
-		perror("open packet device");
+		fprintf(stderr, "pktsetup: Error: Can't open packet device '%s': %s\n", pkt_device, strerror(errno));
 		return 1;
 	}
 
 	if (!rem) {
 		cmd = PACKET_SETUP_DEV;
 		if ((dev_fd = open(device, O_RDONLY | O_NONBLOCK)) == -1) {
-			perror("open cd-rom");
+			fprintf(stderr, "pktsetup: Error: Can't open cd-rom device '%s': %s\n", device, strerror(errno));
 			close(pkt_fd);
 			return 1;
 		}
@@ -117,8 +118,12 @@ static int setup_dev(char *pkt_device, char *device, int rem)
 	}
 		
 	ret = ioctl(pkt_fd, cmd, arg);
-	if (ret == -1)
-		perror("ioctl");
+	if (ret == -1) {
+		if (rem)
+			fprintf(stderr, "pktsetup: Error: Can't tear down packet device '%s': %s\n", pkt_device, strerror(errno));
+		else
+			fprintf(stderr, "pktsetup: Error: Can't setup packet device '%s' for cd-rom device '%s': %s\n", pkt_device, device, strerror(errno));
+	}
 
 	if (dev_fd >= 0)
 		close(dev_fd);
@@ -134,11 +139,15 @@ static int usage(void)
 {
 	printf("pktsetup from " PACKAGE_NAME " " PACKAGE_VERSION "\n");
 	printf("Set up and tear down packet device associations\n");
+#if 0
+	/* pktcdvd >= 0.2.0 is in linux kernel since 2.6.10, so hide help for old interface */
 	printf("For pktcdvd < 0.2.0:\n");
 	printf("  pktsetup /dev/pktcdvd0 /dev/cdrom  setup device\n");
 	printf("  pktsetup -d /dev/pktcdvd0          tear down device\n");
 	printf("For pktcdvd >= 0.2.0:\n");
-	printf("  pktsetup dev_name /dev/cdrom       setup device\n");
+#endif
+	printf("  pktsetup [dev_name] /dev/cdrom     setup device\n");
+	printf("  pktsetup [dev_name] major:minor    setup device\n");
 	printf("  pktsetup -d dev_name               tear down device\n");
 	printf("  pktsetup -d major:minor            tear down device\n");
 	printf("  pktsetup -s                        show device mappings\n");
@@ -168,8 +177,11 @@ static int get_misc_minor(void)
 
 static const char *pkt_dev_name(const char *dev)
 {
-	static char buf[128];
-	snprintf(buf, sizeof(buf), "%s/%s", CTL_DIR, dev);
+	static char buf[512];
+	if (snprintf(buf, sizeof(buf), "%s/%s", CTL_DIR, dev) >= (int)sizeof(buf)) {
+		fprintf(stderr, "pktsetup: Error: Can't process device '%s': %s\n", dev, strerror(ENAMETOOLONG));
+		exit(1);
+	}
 	return buf;
 }
 
@@ -181,12 +193,12 @@ static int create_ctl_dev(void)
 
 	if ((misc_minor = get_misc_minor()) < 0) {
 		if (system("/sbin/modprobe pktcdvd") != 0) {
-			fprintf(stderr, "Can't load pktcdvd kernel module\n");
+			fprintf(stderr, "pktsetup: Error: Can't load pktcdvd kernel module\n");
 		}
 		misc_minor = get_misc_minor();
 	}
 	if (misc_minor < 0) {
-		fprintf(stderr, "Can't find pktcdvd character device\n");
+		fprintf(stderr, "pktsetup: Error: Can't find pktcdvd character device number\n");
 		return -1;
 	}
 	dev = MKDEV(MISC_MAJOR, misc_minor);
@@ -199,7 +211,7 @@ static int create_ctl_dev(void)
 	unlink(pkt_dev_name(CTL_DEV));
 
 	if (mknod(pkt_dev_name(CTL_DEV), S_IFCHR | 0644, dev) < 0) {
-		fprintf(stderr, "Can't create device %s: %s\n", pkt_dev_name(CTL_DEV), strerror(errno));
+		perror("pktsetup: Error: Can't create pktcdvd control device");
 		return -1;
 	}
 
@@ -223,7 +235,7 @@ static int remove_stale_dev_node(int ctl_fd, char *devname)
 		c.command = PKT_CTRL_CMD_STATUS;
 		c.dev_index = i;
 		if (ioctl(ctl_fd, PACKET_CTRL_CMD, &c) < 0) {
-			perror("ioctl");
+			perror("pktsetup: Error: Can't check status of device");
 			return 1;
 		}
 		if (i >= c.num_devices)
@@ -235,6 +247,28 @@ static int remove_stale_dev_node(int ctl_fd, char *devname)
 	return 0;
 }
 
+static dev_t find_pkdev_for_dev(int ctl_fd, dev_t blk_dev)
+{
+	struct pkt_ctrl_command c;
+	unsigned int i;
+
+	memset(&c, 0, sizeof(struct pkt_ctrl_command));
+
+	for (i = 0; ; i++) {
+		c.command = PKT_CTRL_CMD_STATUS;
+		c.dev_index = i;
+		if (ioctl(ctl_fd, PACKET_CTRL_CMD, &c) < 0) {
+			perror("pktsetup: Error: Can't get device mapping");
+			return 0;
+		}
+		if (i >= c.num_devices)
+			return 0;
+		if (c.dev == blk_dev)
+			return c.pkt_dev;
+	}
+	return 0;
+}
+
 static int setup_dev_chardev(char *pkt_device, char *device, int rem)
 {
 	struct pkt_ctrl_command c;
@@ -248,42 +282,50 @@ static int setup_dev_chardev(char *pkt_device, char *device, int rem)
 		return 1;
 
 	if ((ctl_fd = open(pkt_dev_name(CTL_DEV), O_RDONLY)) < 0) {
-		perror("ctl open");
+		perror("pktsetup: Error: Can't open pktcdvd control device");
 		return 1;
 	}
 
 	if (!rem) {
 		if ((dev_fd = open(device, O_RDONLY | O_NONBLOCK)) == -1) {
-			perror("open cd-rom");
-			goto out_close;
-		}
-		if (init_cdrom(dev_fd)) {
+			int major, minor, old_errno;
+			old_errno = errno;
+			if (sscanf(device, "%d:%d", &major, &minor) != 2) {
+				fprintf(stderr, "pktsetup: Error: Can't open cd-rom device '%s': %s\n", device, strerror(old_errno));
+				goto out_close;
+			}
+			c.dev = MKDEV(major, minor);
+		} else {
+			if (fstat(dev_fd, &stat_buf) < 0) {
+				fprintf(stderr, "pktsetup: Error: Can't stat cd-rom device '%s': %s\n", device, strerror(errno));
+				close(dev_fd);
+				goto out_close;
+			}
+			if (!S_ISBLK(stat_buf.st_mode)) {
+				fprintf(stderr, "pktsetup: Error: Node '%s' is not a block device\n", device);
+				close(dev_fd);
+				goto out_close;
+			}
+			if (init_cdrom(dev_fd)) {
+				close(dev_fd);
+				goto out_close;
+			}
 			close(dev_fd);
-			goto out_close;
+			c.dev = stat_buf.st_rdev;
 		}
-		close(dev_fd);
 
-		if (stat(device, &stat_buf) < 0) {
-			perror("stat cd-rom");
-			goto out_close;
-		}
-		if (!S_ISBLK(stat_buf.st_mode)) {
-			fprintf(stderr, "Not a block device\n");
-			goto out_close;
-		}
 		c.command = PKT_CTRL_CMD_SETUP;
-		c.dev = stat_buf.st_rdev;
 
-		if (remove_stale_dev_node(ctl_fd, pkt_device) != 0) {
-			fprintf(stderr, "Device node '%s' already in use\n", pkt_device);
+		if (pkt_device && remove_stale_dev_node(ctl_fd, pkt_device) != 0) {
+			fprintf(stderr, "pktsetup: Error: Device node '%s' already in use\n", pkt_device);
 			goto out_close;
 		}
 		if (ioctl(ctl_fd, PACKET_CTRL_CMD, &c) < 0) {
-			perror("ioctl");
+			fprintf(stderr, "pktsetup: Error: Can't setup packet device for cd-rom device '%s': %s\n", device, strerror(errno));
 			goto out_close;
 		}
-		if (mknod(pkt_dev_name(pkt_device), S_IFBLK | 0640, c.pkt_dev) < 0) {
-			fprintf(stderr, "Can't create device node '%s': %s\n", pkt_dev_name(pkt_device), strerror(errno));
+		if (pkt_device && mknod(pkt_dev_name(pkt_device), S_IFBLK | 0640, c.pkt_dev) < 0 && errno != EEXIST) {
+			fprintf(stderr, "pktsetup: Error: Can't create device node '%s': %s\n", pkt_device, strerror(errno));
 			goto out_close;
 		}
 		ret = 0;
@@ -294,22 +336,38 @@ static int setup_dev_chardev(char *pkt_device, char *device, int rem)
 		    S_ISBLK(stat_buf.st_mode)) {
 			major = MAJOR(stat_buf.st_rdev);
 			minor = MINOR(stat_buf.st_rdev);
-			remove_node = 1;
+			if (strncmp(pkt_device, "pktcdvd", sizeof("pktcdvd")-1) == 0)
+				remove_node = 0;
+			else
+				remove_node = 1;
 		} else if (sscanf(pkt_device, "%d:%d", &major, &minor) == 2) {
 			remove_node = 0;
 		} else {
-			fprintf(stderr, "Can't find major/minor numbers\n");
+			fprintf(stderr, "pktsetup: Error: Can't find major/minor numbers for packet device '%s'\n", pkt_device);
 			goto out_close;
 		}
 
 		c.command = PKT_CTRL_CMD_TEARDOWN;
 		c.pkt_dev = MKDEV(major, minor);
 		if (ioctl(ctl_fd, PACKET_CTRL_CMD, &c) < 0) {
-			perror("ioctl");
-			goto out_close;
+			int cur_errno = errno;
+			if (cur_errno != ENXIO)
+				goto out_error;
+			remove_node = 0;
+			c.pkt_dev = find_pkdev_for_dev(ctl_fd, MKDEV(major, minor));
+			if (!c.pkt_dev)
+				goto out_error;
+			if (ioctl(ctl_fd, PACKET_CTRL_CMD, &c) < 0) {
+				cur_errno = errno;
+out_error:
+				fprintf(stderr, "pktsetup: Error: Can't tear down packet device '%s': %s\n", pkt_device, strerror(cur_errno));
+				goto out_close;
+			}
 		}
 		if (remove_node) {
-			if (unlink(pkt_dev_name(pkt_device)) == 0)
+			if (unlink(pkt_dev_name(pkt_device)) != 0)
+				fprintf(stderr, "pktsetup: Error: Can't unlink device node '%s': %s\n", pkt_device, strerror(errno));
+			else
 				ret = 0;
 		} else {
 			ret = 0;
@@ -321,27 +379,30 @@ out_close:
 	return ret;
 }
 
-static void show_mappings(void)
+static int show_mappings(void)
 {
 	struct pkt_ctrl_command c;
 	unsigned int i;
 	int ctl_fd;
+	int ret;
 
+	ret = 0;
 	memset(&c, 0, sizeof(struct pkt_ctrl_command));
 
 	if (create_ctl_dev() < 0)
-		return;
+		return 1;
 
 	if ((ctl_fd = open(pkt_dev_name(CTL_DEV), O_RDONLY)) < 0) {
-		perror("ctl open");
-		return;
+		perror("pktsetup: Error: Can't open pktcdvd control device");
+		return 1;
 	}
 
 	for (i = 0; ; i++) {
 		c.command = PKT_CTRL_CMD_STATUS;
 		c.dev_index = i;
 		if (ioctl(ctl_fd, PACKET_CTRL_CMD, &c) < 0) {
-			perror("ioctl");
+			perror("pktsetup: Error: Can't show device mapping");
+			ret = 1;
 			goto out_close;
 		}
 		if (i >= c.num_devices)
@@ -355,11 +416,13 @@ static void show_mappings(void)
 
 out_close:
 	close(ctl_fd);
+
+	return ret;
 }
 
 int main(int argc, char **argv)
 {
-	int rem = 0, c;
+	int rem = 0, c, old_api;
 	char *pkt_device;
 	char *device;
 
@@ -372,20 +435,26 @@ int main(int argc, char **argv)
 				rem = 1;
 				break;
 			case 's':
-				show_mappings();
-				return 0;
+				return show_mappings();
 			default:
 				return usage();
 		}
 	}
-	if (optind == argc || (!rem && optind+1 == argc) || (rem && optind+1 < argc) || (!rem && optind+2 < argc))
+
+	if (optind == argc || (!rem && optind+2 < argc) || (rem && optind+1 < argc))
 		return usage();
-	pkt_device = argv[optind];
-	if (!rem)
-		device = argv[optind + 1];
-	else
-		device = NULL;
-	if (strchr(pkt_device, '/'))
+
+	if (!rem && optind+1 == argc) {
+		device = argv[optind];
+		pkt_device = NULL;
+		old_api = 0;
+	} else {
+		pkt_device = argv[optind];
+		device = rem ? NULL : argv[optind + 1];
+		old_api = (strchr(pkt_device, '/') != NULL);
+	}
+
+	if (old_api)
 		return setup_dev(pkt_device, device, rem);
 	else
 		return setup_dev_chardev(pkt_device, device, rem);
